@@ -674,7 +674,7 @@ class GeoArray(object):
             arr = self[:, :, fromBand] if self.ndim == 3 and fromBand is not None else self[:]
 
             if self.nodata is None:
-                mask = np.ones((self.rows, self.cols), np.bool)
+                mask = np.ones((self.rows, self.cols), bool)
 
             elif np.isnan(self.nodata):
                 nanmask = np.isnan(arr)
@@ -818,7 +818,6 @@ class GeoArray(object):
         :param getitem_params:  <list> a list of slices in the form [row_slice, col_slice, band_slice]
         :return out_arr:        <np.ndarray> the output array
         """
-
         ds = gdal.Open(path)
         if not ds:
             raise Exception('Error reading file:  ' + gdal.GetLastErrorMsg())
@@ -971,7 +970,7 @@ class GeoArray(object):
             # write _arr_cache
             self._arr_cache = dict(pos=arr_pos, arr_cached=out_arr)
 
-        return out_arr  # TODO implement check of returned datatype (e.g. NoDataMask should always return np.bool
+        return out_arr  # TODO implement check of returned datatype (e.g. NoDataMask should always return bool
         # TODO -> would be np.int8 if an int8 file is read from disk
 
     def save(self, out_path, fmt='ENVI', creationOptions=None):
@@ -1022,16 +1021,34 @@ class GeoArray(object):
             # ENVI #
             ########
             if fmt == 'ENVI':
-                ds_out.SetMetadata(envi_metadict, 'ENVI')
+                for bidx in range(self.bands):
+                    band = ds_out.GetRasterBand(bidx + 1)
 
-                if 'band_names' in envi_metadict:
-                    for bidx in range(self.bands):
-                        band = ds_out.GetRasterBand(bidx + 1)
+                    if 'band_names' in envi_metadict:
                         bandname = self.metadata.band_meta['band_names'][bidx].strip()
                         band.SetDescription(bandname)
-
                         assert band.GetDescription() == bandname
-                        del band
+
+                    # NOTE: band-specific nodata values are not supported by the ENVI header format
+                    if 'nodata' in envi_metadict:
+                        nodataVal = self.metadata.band_meta['nodata'][bidx]
+                        band.SetNoDataValue(nodataVal)
+                        assert band.GetNoDataValue() == nodataVal
+
+                    del band
+
+                # avoid that band names are written to global meta
+                if 'band_names' in envi_metadict:
+                    del envi_metadict['band_names']
+
+                if 'nodata' in envi_metadict:
+                    del envi_metadict['nodata']
+
+                # set data ignore value in case self.metadata.band_meta contains a unique nodata value
+                if 'nodata' in self.metadata.band_meta and len(set(self.metadata.band_meta['nodata'])) == 1:
+                    envi_metadict['data ignore value'] = str(self.metadata.band_meta['nodata'][0])
+
+                ds_out.SetMetadata(envi_metadict, 'ENVI')
 
                 if 'description' in envi_metadict:
                     ds_out.SetDescription(envi_metadict['description'])
@@ -1052,12 +1069,22 @@ class GeoArray(object):
 
                 for bidx in range(self.bands):
                     band = ds_out.GetRasterBand(bidx + 1)
-                    bandmeta = bandmeta_dict[bidx]
-                    # meta2write = dict((k, repr(v)) for k, v in self.metadata.band_meta.items() if v is not np.nan)
-                    band.SetMetadata(bandmeta)
+                    bandmeta = bandmeta_dict[bidx].copy()
 
-                    if 'band_names' in envi_metadict:
+                    # filter global metadata out
+                    bandmeta = {k: v for k, v in bandmeta.items() if k not in self.metadata.global_meta}
+                    # meta2write = dict((k, repr(v)) for k, v in self.metadata.band_meta.items() if v is not np.nan)
+
+                    if 'band_names' in bandmeta:
                         band.SetDescription(self.metadata.band_meta['band_names'][bidx].strip())
+                        del bandmeta['band_names']
+
+                    if 'nodata' in bandmeta:
+                        band.SetNoDataValue(self.metadata.band_meta['nodata'][bidx])
+                        del bandmeta['nodata']
+
+                    if bandmeta:
+                        band.SetMetadata(bandmeta)
 
                     band.FlushCache()
                     del band
@@ -1156,7 +1183,7 @@ class GeoArray(object):
         palette = copy(palette)  # allows to modify the colormap as in-place modifications are not allowed anymore
 
         if nodataVal is not None and \
-           np.std(image2plot.astype(np.float)) != 0:  # do not show nodata; float is needed to avoid overflow error
+           np.std(image2plot.astype(float)) != 0:  # do not show nodata; float is needed to avoid overflow error
 
             image2plot = np.ma.masked_equal(image2plot, nodataVal)
             vmin_auto = np.nanpercentile(image2plot.compressed(), pmin)
@@ -1210,11 +1237,15 @@ class GeoArray(object):
         # get image to plot
         nodataVal = nodataVal if nodataVal is not None else self.nodata if self.nodata is not None else -9999
         image2plot, gt, prj = \
-            self._get_plottable_image(xlim, ylim, band, boundsMap=boundsMap, boundsMapPrj=boundsMapPrj,
-                                      res_factor=res_factor, nodataVal=nodataVal, ignore_rotation=ignore_rotation)
+            self._get_plottable_image(xlim, ylim, band,
+                                      boundsMap=boundsMap,
+                                      boundsMapPrj=boundsMapPrj,
+                                      res_factor=res_factor,
+                                      nodataVal=nodataVal,
+                                      ignore_rotation=ignore_rotation)
 
         palette, vmin, vmax = self._get_cmap_vmin_vmax(cmap, vmin, vmax, pmin, pmax, image2plot, nodataVal)
-        if nodataVal is not None and np.std(image2plot.astype(np.float)) != 0:
+        if nodataVal is not None and np.std(image2plot.astype(float)) != 0:
             image2plot = np.ma.masked_equal(image2plot, nodataVal)
 
         # check availability of holoviews
@@ -1236,12 +1267,16 @@ class GeoArray(object):
 
             def get_hv_image(b):
                 # FIXME ylabels have the wrong order
-                return hv.Image(image2plot[:, :, b] if b is not None else image2plot,
-                                bounds=(cS, rS, cE, rE))(
-                    style={'cmap': 'gray'}, plot={'fig_inches': 4 if figsize is None else figsize, 'show_grid': True})
+                hv_image = hv.Image(image2plot[:, :, b] if b is not None else image2plot,
+                                    bounds=(cS, rS, cE, rE))
+                return hv_image.opts(style={'cmap': 'gray'},
+                                     plot={'fig_inches': 4 if figsize is None else figsize,
+                                           'show_grid': True})
 
             # hvIm = hv.Image(image2plot)(style={'cmap': 'gray'}, figure_inches=figsize)
-            hmap = hv.HoloMap([(band, get_hv_image(band)) for band in range(image2plot.shape[2])], kdims=['band'])
+            hmap = hv.HoloMap([(band, get_hv_image(band))
+                               for band in range(image2plot.shape[2])],
+                              kdims=['band'])
 
             return hmap
 
@@ -1256,12 +1291,13 @@ class GeoArray(object):
                 ax = plt.gca()
 
             rows, cols = image2plot.shape[:2]
-            im = ax.imshow(image2plot.astype(np.int),
+            im = ax.imshow(image2plot.astype(int),
                            palette,
                            interpolation=interpolation,
                            extent=(0, cols, rows, 0),
                            vmin=vmin,
-                           vmax=vmax, )  # compressed excludes nodata values
+                           vmax=vmax
+                           )  # compressed excludes nodata values
             plt.show()
 
             return im
